// This file is part of the Acts project.
//
// Copyright (C) 2019 CERN for the benefit of the Acts project
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

#pragma once

#include <limits>

#include "Acts/Material/Material.hpp"

namespace Acts {

/// Accumulate and average volume-based material properties.
///
/// This class is intended to be used during the mapping process.
class AccumulatedVolumeMaterial {
 public:
  /// Add one entry with the given material properties.
  void accumulate(const Material& mat);

  /// Compute the average material collected so far.
  ///
  /// @returns Vacuum properties if no matter has been accumulated yet.
  Material average();

 private:
  float m_totalX0{std::numeric_limits<float>::infinity()};
  float m_totalL0{std::numeric_limits<float>::infinity()};
  float m_totalAr{0.};
  float m_totalZ{0.};
  float m_totalRho{0.};
  unsigned int m_materialEntries{0};
  unsigned int m_vacuumEntries{0};
};

}  // namespace Acts
